import { __extends } from 'tslib';
import * as React from 'react';
import { useId, useRef, useInsertionEffect } from 'react';

/**
 * Measurement functionality has to be within a separate component
 * to leverage snapshot lifecycle.
 */
var PopChildMeasure = /** @class */ (function (_super) {
    __extends(PopChildMeasure, _super);
    function PopChildMeasure() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PopChildMeasure.prototype.getSnapshotBeforeUpdate = function (prevProps) {
        var element = this.props.childRef.current;
        if (element && prevProps.isPresent && !this.props.isPresent) {
            var position = getComputedStyle(element).position;
            var size = this.props.sizeRef.current;
            size.height = element.offsetHeight || 0;
            size.width = element.offsetWidth || 0;
            size.top = size.left = "auto";
            /**
             * If this element is position: relative and the parent has
             * padding, we need to explicitly set a top and/or left.
             */
            if (position === "relative" && element.offsetParent) {
                var _a = getComputedStyle(element.offsetParent), paddingTop = _a.paddingTop, paddingLeft = _a.paddingLeft;
                if (parseFloat(paddingTop)) {
                    size.top = "".concat(element.offsetTop, "px");
                }
                if (parseFloat(paddingLeft)) {
                    size.left = "".concat(element.offsetLeft, "px");
                }
            }
        }
        return null;
    };
    /**
     * Required with getSnapshotBeforeUpdate to stop React complaining.
     */
    PopChildMeasure.prototype.componentDidUpdate = function () { };
    PopChildMeasure.prototype.render = function () {
        return this.props.children;
    };
    return PopChildMeasure;
}(React.Component));
function PopChild(_a) {
    var children = _a.children, isPresent = _a.isPresent;
    var id = useId();
    var ref = useRef(null);
    var size = useRef({
        width: 0,
        height: 0,
        top: "auto",
        left: "auto",
    });
    /**
     * We create and inject a style block so we can apply this explicit
     * sizing in a non-destructive manner by just deleting the style block.
     *
     * We can't apply size via render as the measurement happens
     * in getSnapshotBeforeUpdate (post-render), likewise if we apply the
     * styles directly on the DOM node, we might be overwriting
     * styles set via the style prop.
     */
    useInsertionEffect(function () {
        var _a;
        var _b = size.current, width = _b.width, height = _b.height, top = _b.top, left = _b.left;
        if (isPresent || !ref.current || !width || !height)
            return;
        ref.current.dataset.motionPopId = id;
        var style = document.createElement("style");
        document.head.appendChild(style);
        (_a = style.sheet) === null || _a === void 0 ? void 0 : _a.insertRule("\n          [data-motion-pop-id=\"".concat(id, "\"] {\n            position: absolute !important;\n            width: ").concat(width, "px !important;\n            height: ").concat(height, "px !important;\n            top: ").concat(top, " !important;\n            left: ").concat(left, " !important;\n          }\n        "));
        return function () {
            document.head.removeChild(style);
        };
    }, [isPresent]);
    return (React.createElement(PopChildMeasure, { isPresent: isPresent, childRef: ref, sizeRef: size }, React.cloneElement(children, { ref: ref })));
}

export { PopChild };
